import os
import torch
import logging

from torchfly.flylogger import FlyLogger
from torchfly.flyconfig import FlyConfig
from torchfly.training import Trainer
import torchfly.distributed as distributed
from torchfly.utilities import set_random_seed
from omegaconf import OmegaConf

from memformers.models.memformerC2_base.memformer_flymodel import MemformerFlyModel
from memformers.recurrent_training.recurrent_trainer import RecurrentTrainer
from dataloader import DataLoaderHelper

logger = logging.getLogger(__name__)

def main():
    config = FlyConfig.load("config/memformerC2_base_denoising.yaml")
    set_random_seed(config.training.random_seed + int(os.environ.get("RANK", 0)))

    data_helper = DataLoaderHelper(config.data_processing, training_batch_size=config.training.batch_size, training_num_batch_chunks=config.training.num_batch_chunks)
    train_dataloader = data_helper.train_dataloader_fn()

    model = MemformerFlyModel(config)
    model.configure_metrics()

    trainer = RecurrentTrainer(config.training, model)

    with FlyLogger(config.flylogger) as flylogger:
        # save_config
        with open("config.yaml", "w") as f:
            OmegaConf.save(config, f)
        
        logger.info(model.model.recurrent_training_cell.model_config)
        logger.info(config)

        trainer.train(config.training, train_dataloader)


if __name__ == "__main__":
    main()
